<?php

namespace Drupal\smart_content_block\Plugin\Block;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\smart_content\Entity\SmartVariationSet;
use Drupal\smart_content\Form\SmartVariationSetForm;

/**
 * Provides a 'SmartBlock' block.
 *
 * @Block(
 *  id = "smart_block",
 *  admin_label = @Translation("Smart block"),
 * )
 */
class SmartBlock extends BlockBase {
  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'variation_set' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {


    $configuration = $this->getConfiguration();
    // @todo: make this entire section more pluggable.
    /** @var SmartVariationSet $entity */
    $entity = \Drupal::entityTypeManager()
      ->getStorage('smart_variation_set')->load($configuration['variation_set']);

    return [
      'smart_content' => $entity->renderPlaceholder()
    ];
  }

  public function blockForm($form, FormStateInterface $form_state) {
    $form['#process'][] = [$this, 'buildWidget'];
    return $form;
  }


  /**
   * Render API callback: builds the formatter settings elements.
   */
  public function buildWidget(array &$element, FormStateInterface $form_state, array &$complete_form) {
    $configuration = $this->getConfiguration();


    if(!$entity = SmartVariationSetForm::getEntityState($form_state, $element['#array_parents'])) {
      if(!empty($configuration['variation_set'])) {
        $entity = \Drupal::entityTypeManager()
          ->getStorage('smart_variation_set')->load($configuration['variation_set']);
      }
      else {
        $entity = \Drupal::entityTypeManager()
          ->getStorage('smart_variation_set')
          ->create([]);
      }
      $smart_variation_set_type = \Drupal::service('plugin.manager.smart_content.variation_set_type')->createInstance('block');
      $smart_variation_set_type->entity = $entity;
      $entity->setVariationSetType($smart_variation_set_type);
    }

    SmartVariationSetForm::saveEntityState($form_state, $element['#array_parents'], $entity);

    $variation_set_form = [];

    if(!empty($element['variation_set_config'])) {
      $variation_set_form = $element['variation_set_config'];
    }

    $variation_set_form_state = SubformState::createForSubform($variation_set_form, $element, $form_state);
    $variation_set_form = \Drupal::entityTypeManager()->getFormObject('smart_variation_set', 'edit')->setEntity($entity)->buildForm($variation_set_form, $variation_set_form_state);

    $element['variation_set_config'] = $variation_set_form;

    return $element;

  }
  /**
  * {@inheritdoc}
   * @todo: blockSubmit doesn't track #array_parents the same way when embedded,
   * @todo: Need to find best way to find form parents consistently.
  */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $variation_set_form = [];
    $subform = $form_state->getCompleteForm();
    $parents = $form['#array_parents'];
    $parents[] = 'variation_set_config';

    //@todo: Find better way to do this.
    if(!empty($form ['settings']['variation_set_config'])) {
      $variation_set_form = $form['settings']['variation_set_config'];
    }
    else if($variation_set_form = NestedArray::getValue($subform, $parents)) {

    } else {
      $variation_set_form = [];
    }

    $variation_set_form_state = SubformState::createForSubform($variation_set_form, $subform, $form_state->getCompleteFormState());
    //@todo: Find better way to do this.
    if(isset($form['#form_id']) && $form['#form_id'] == 'block_form') {
      $entity = SmartVariationSetForm::getEntityState($form_state, ['settings']);
    } else {
      $entity = SmartVariationSetForm::getEntityState($form_state, $form['#array_parents']);
    }
    //@todo: determine best way to set entity.
    if($entity) {
      \Drupal::entityTypeManager()->getFormObject('smart_variation_set', 'edit')->setEntity($entity)->submitForm($variation_set_form, $variation_set_form_state);
      $this->setConfigurationValue('variation_set', $entity->id());
    }

  }
}
