<?php

namespace Drupal\smart_content_block\Plugin\smart_content\VariationSetType;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\smart_content\Form\SmartVariationSetForm;
use Drupal\smart_content\VariationSetType\VariationSetTypeBase;

/**
 * @SmartVariationSetType(
 *   id = "block",
 *   label = @Translation("Block"),
 * )
 */
class Block extends VariationSetTypeBase {

  /**
   * @inheritdoc
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['#process'][] = [$this, 'buildWidget'];
    return $form;
  }

  public function buildWidget(array &$element, FormStateInterface $form_state, array &$complete_form) {
    //@todo: determine better way to do ID
    $wrapper_id = $element['#id'] . '-variation-set-wrapper';
    $wrapper_items_id = $element['#id'] . '-variation-set-items-wrapper';

    // TODO: Implement buildConfigurationForm() method.
    $element['variations_config'] = [
      '#type' => 'container',
      '#title' => 'Variations',
      '#prefix' => '<div id="' . $wrapper_id . '" class="variations-container">',
      '#suffix' => '</div>',
      '#tree' => TRUE,
    ];

    $element['variations_config']['variation_items'] = [
      '#type' => 'table',
      '#header' => [t('Variations'), t('Weight'), t('')],
      '#prefix' => '<div id="' . $wrapper_items_id . '" class="variations-container-items">',
      '#suffix' => '</div>',
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => $wrapper_items_id . '-order-weight',
        ],
      ],
    ];


    $i = 0;
    foreach ($this->entity->getVariations() as $variation_id => $variation) {

      $i++;
      SmartVariationSetForm::pluginForm($variation, $element, $form_state, [
        'variations_config',
        'variation_items',
        $variation_id,
        'plugin_form',
      ]);
      //
      $element['variations_config']['variation_items'][$variation_id]['plugin_form']['#type'] = 'fieldset';
      $element['variations_config']['variation_items'][$variation_id]['plugin_form']['#title'] = 'Variation ' . $i;
      $element['variations_config']['variation_items'][$variation_id]['plugin_form']['#attributes']['class'][] = 'variation-container';
      //  $form['variations_config']['variation_items'][$variation_id]['plugin_form']['#open'] = $this->entity->isNew();


      $element['variations_config']['variation_items'][$variation_id]['#attributes']['class'][] = 'draggable';
      $element['variations_config']['variation_items'][$variation_id]['#weight'] = $variation->getWeight();

      //  $form['variations_config']['variation_items'][$variation_id]['plugin_form'] = $variation_form;

      $element['variations_config']['variation_items'][$variation_id]['weight'] = [
        '#type' => 'weight',
        '#title' => 'Weight',
        '#title_display' => 'invisible',
        // #default_value' => $entity->get('weight'),
        // Classify the weight element for #tabledrag.
        '#attributes' => ['class' => [$wrapper_items_id . '-order-weight']],
      ];

      $element['variations_config']['variation_items'][$variation_id]['remove_variation'] = [
        '#type' => 'submit',
        '#value' => t('Remove Variation'),
        '#name' => 'remove_variation__' . $variation_id,
        '#submit' => [[$this, 'removeElementVariation']],
        '#attributes' => ['class' => ['align-right']],
        '#ajax' => [
          'callback' => [$this, 'removeElementVariationAjax'],
          'wrapper' => $wrapper_id,
        ],
        '#limit_validation_errors' => [],
      ];
    }

    $element['variations_config']['add_variation'] = [
      '#type' => 'container',
    ];

    $element['variations_config']['add_variation'] = [
      '#type' => 'submit',
      '#value' => t('Add Variation'),
      '#submit' => [[$this, 'addElementVariation']],
      '#ajax' => [
        'callback' => [$this, 'addElementVariationAjax'],
        'wrapper' => $wrapper_id,
      ],
      '#limit_validation_errors' => [],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->attachTableWeight($form_state->getValues()['variations_config']['variation_items']);
    foreach ($this->entity->getVariations() as $variation_id => $variation) {
      SmartVariationSetForm::pluginFormSubmit($variation, $form, $form_state, [
        'variations_config',
        'variation_items',
        $variation_id,
        'plugin_form',
      ]);
    }
    $this->setConfiguration($form_state->getValues());
  }

  // @todo: determine a better way to do this.
  public function attachTableWeight($values) {
    foreach ($this->entity->getVariations() as $variation) {
      if (isset($values[$variation->id()]['weight'])) {
        $variation->setWeight($values[$variation->id()]['weight']);
      }
    }
    $this->entity->sortVariations();
  }

  public function addElementVariation(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $items = array_slice($button['#parents'], 0, -1);
    $items[] = 'variation_items';
    $values = NestedArray::getValue($form_state->getUserInput(), $items);
    $this->attachTableWeight($values);
    $this->entity->addVariation(\Drupal::service('plugin.manager.smart_content.variation')
      ->createInstance('variation_standard'));
    $form_state->setRebuild();
  }

  public function addElementVariationAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -1));
  }

  public function removeElementVariation(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $items = array_slice($button['#parents'], 0, -3);
    $items[] = 'variation_items';
    $values = NestedArray::getValue($form_state->getUserInput(), $items);
    $this->attachTableWeight($values);
    list($action, $name) = explode('__', $form_state->getTriggeringElement()['#name']);
    $this->entity->removeVariation($name);
    $form_state->setRebuild();
  }

  public function removeElementVariationAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -3));
  }

}
