<?php

namespace Drupal\smart_content\Plugin\smart_content\Variation;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\smart_content\Annotation\SmartVariation;
use Drupal\smart_content\Condition\ConditionManager;
use Drupal\smart_content\Form\SmartVariationSetForm;
use Drupal\smart_content\Reaction\ReactionManager;
use Drupal\smart_content\Variation\VariationBase;

/**
 * Provides a default Smart Condition.
 *
 * @SmartVariation(
 *   id = "variation_standard",
 *   label = @Translation("Standard Variation"),
 * )
 */
class VariationStandard extends VariationBase {

  /**
   * @inheritdoc
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['#process'][] = [$this, 'buildWidget'];
    return $form;
  }


  /**
   * Render API callback: builds the formatter settings elements.
   */
  public function buildWidget(array &$element, FormStateInterface $form_state, array &$complete_form) {
    //@todo: determine better way to do ID
    $wrapper_id = $element['#id'] . '-variation-set-wrapper';
    $wrapper_items_id = $element['#id'] . '-variation-set-items-wrapper';

    $element['conditions_config'] = [
      '#type' => 'container',
      '#title' => 'Conditions',
      '#prefix' => '<div id="' . $wrapper_id . '-conditions' . '" class="conditions-container variation-conditions-container">',
      '#suffix' => '</div>',
      '#tree' => TRUE,
    ];

    $element['conditions_config']['condition_items'] = [
      '#type' => 'table',
      '#header' => [t('Condition(s)'), t('Weight'), ''],
      '#prefix' => '<div id="' . $wrapper_items_id . '-conditions' . '" class="conditions-container-items variation-conditions-containers-items">',
      '#suffix' => '</div>',
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => $wrapper_items_id . '-order-weight',
        ],
      ],
    ];

    //@todo: add draggable weights
    foreach ($this->getConditions() as $condition_id => $condition) {
      if ($condition instanceof PluginFormInterface) {
        SmartVariationSetForm::pluginForm($condition, $element, $form_state, [
          'conditions_config',
          'condition_items',
          $condition_id,
          'plugin_form',
        ]);

        $element['conditions_config']['condition_items'][$condition_id]['plugin_form']['#type'] = 'container';
        $element['conditions_config']['condition_items'][$condition_id]['plugin_form']['#title'] = $condition->getPluginId();
        $element['conditions_config']['condition_items'][$condition_id]['plugin_form']['#attributes']['class'][] = 'condition';
        $element['conditions_config']['condition_items'][$condition_id]['#weight'] = $condition->getWeight();

        $element['conditions_config']['condition_items'][$condition_id]['#attributes']['class'][] = 'draggable';
        //        $element['conditions_config']['condition_items'][$condition_id]['#attributes']['class'][] = 'row-condition';

        $element['conditions_config']['condition_items'][$condition_id]['weight'] = [
          '#type' => 'weight',
          '#title' => 'Weight',
          '#title_display' => 'invisible',
          '#default_value' => $condition->getWeight(),
          // Classify the weight element for #tabledrag.
          '#attributes' => ['class' => [$wrapper_items_id . '-order-weight']],
        ];

        //@todo: Fix bug where this fails to work first time.
        $element['conditions_config']['condition_items'][$condition_id]['remove_condition'] = [
          '#type' => 'submit',
          '#value' => t('Remove Condition'),
          '#name' => 'remove_condition_' . $this->id() . '__' . $condition_id,
          '#submit' => [[$this, 'removeElementCondition']],
          '#attributes' => ['class' => ['align-right']],
          '#ajax' => [
            'callback' => [$this, 'removeElementConditionAjax'],
            'wrapper' => $wrapper_id . '-conditions',
          ],
          '#limit_validation_errors' => [],
        ];
      }
      //
    }


    //@todo: move condition type select condition manager
    $element['conditions_config']['add_condition'] = [
      '#type' => 'container',
      '#title' => 'Add Condition',
      '#attributes' => ['class' => ['condition-add-container']],
    ];
    $element['conditions_config']['add_condition']['condition_type'] = [
      '#title' => 'Condition Type',
      '#title_display' => 'invisible',
      '#type' => 'select',
      '#options' => $this->conditionManager->getFormOptions(),
      '#empty_value' => '',
    ];
    $element['conditions_config']['add_condition']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Add Condition'),
      '#validate' => [[$this, 'addElementConditionValidate']],
      '#submit' => [[$this, 'addElementCondition']],
      '#name' => 'add_condition_' . $this->id(),
      '#ajax' => [
        'callback' => [$this, 'addElementConditionAjax'],
        'wrapper' => $wrapper_id . '-conditions',
      ],
      '#limit_validation_errors' => [
        array_merge($element['#array_parents'], [
          'conditions_config',
          'add_condition',
          'condition_type',
        ]),
      ],
    ];

    $element['reactions_config'] = [
      '#type' => 'container',
      '#title' => 'Reactions',
      '#prefix' => '<div id="' . $wrapper_id . '-reactions' . '">',
      '#suffix' => '</div>',
      '#tree' => TRUE,
    ];

    $element['reactions_config']['reaction_items'] = [
      '#type' => 'table',
      '#header' => [t('Reaction(s)'), t('Weight'), ''],
      '#prefix' => '<div id="' . $wrapper_items_id . '-reactions' . '">',
      '#suffix' => '</div>',
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => $wrapper_items_id . '-order-reaction-weight',
        ],
      ],
    ];

    //@todo: add draggable weights
    foreach ($this->getReactions() as $reaction_id => $reaction) {
      if ($reaction instanceof PluginFormInterface) {
        SmartVariationSetForm::pluginForm($reaction, $element, $form_state, [
          'reactions_config',
          'reaction_items',
          $reaction_id,
          'plugin_form',
        ]);

        $element['reactions_config']['reaction_items'][$reaction_id]['plugin_form']['#type'] = 'container';
        $element['reactions_config']['reaction_items'][$reaction_id]['plugin_form']['#title'] = $reaction->getPluginId();
        $element['reactions_config']['reaction_items'][$reaction_id]['plugin_form']['#attributes']['class'][] = 'reaction';
        $element['reactions_config']['reaction_items'][$reaction_id]['#weight'] = $reaction->getWeight();


        $element['reactions_config']['reaction_items'][$reaction_id]['#attributes']['class'][] = 'draggable';
        $element['reactions_config']['reaction_items'][$reaction_id]['#attributes']['class'][] = 'row-reaction';

        $element['reactions_config']['reaction_items'][$reaction_id]['weight'] = [
          '#type' => 'weight',
          '#title' => 'Weight',
          '#title_display' => 'invisible',
          '#default_value' => $reaction->getWeight(),
          // Classify the weight element for #tabledrag.
          '#attributes' => ['class' => [$wrapper_items_id . '-order-reaction-weight']],
        ];


        $element['reactions_config']['reaction_items'][$reaction_id]['remove_reaction'] = [
          '#type' => 'submit',
          '#value' => t('Remove Reaction'),
          '#name' => 'remove_reaction_' . $this->id() . '__' . $reaction_id,
          '#submit' => [[$this, 'removeElementReaction']],
          '#attributes' => ['class' => ['align-right']],
          '#ajax' => [
            'callback' => [$this, 'removeElementReactionAjax'],
            'wrapper' => $wrapper_id . '-reactions',
          ],
          '#limit_validation_errors' => [],
        ];
      }
      //
    }

    $element['reactions_config']['add_reaction'] = [
      '#type' => 'container',
    ];


    //@todo: move reaction type select reaction manager
    $element['reactions_config']['add_reaction'] = [
      '#type' => 'container',
      '#title' => 'Add Reaction',
    ];

    //    $element['reactions_config']['add_reaction']['reaction_type'] = [
    //          '#title' => 'Reaction Type',
    //          '#title_display' => 'invisible',
    //          '#type' => 'select',
    //          '#options' => $this->reactionManager->getFormOptions(),
    //        ];
    //@todo: allow to work with multiple reaction types
    $element['reactions_config']['add_reaction']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Add Reaction'),
      '#submit' => [[$this, 'addElementReaction']],
      '#name' => 'add_reaction_' . $this->id(),
      '#ajax' => [
        'callback' => [$this, 'addElementReactionAjax'],
        'wrapper' => $wrapper_id . '-reactions',
      ],
      '#limit_validation_errors' => []
      //            '#limit_validation_errors' => [array_merge(SmartVariationSetForm::getPluginFormParents($form), ['reactions_config', 'add_reaction', 'reaction_type'])]
    ];


    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    //@todo: add validation for add condition
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    self::attachTableConditionWeight($form_state->getValues()['conditions_config']['condition_items']);
    self::attachTableReactionWeight($form_state->getValues()['reactions_config']['reaction_items']);
    foreach ($this->getConditions() as $condition_id => $condition) {
      SmartVariationSetForm::pluginFormSubmit($condition, $form, $form_state, [
        'conditions_config',
        'condition_items',
        $condition_id,
        'plugin_form',
      ]);
    }
    foreach ($this->getReactions() as $reaction_id => $reaction) {
      if ($reaction instanceof PluginFormInterface) {
        SmartVariationSetForm::pluginFormSubmit($reaction, $form, $form_state, [
          'reactions_config',
          'reaction_items',
          $reaction_id,
          'plugin_form',
        ]);
      }
    }
  }


  // @todo: determine if there is a better way to do this. copy better way to block form.
  public function attachTableConditionWeight($values) {
    foreach ($this->getConditions() as $condition) {
      if (isset($values[$condition->id()]['weight'])) {
        $condition->setWeight($values[$condition->id()]['weight']);
      }
    }
    $this->sortConditions();
  }

  public function addElementConditionValidate(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $parents = array_slice($button['#parents'], 0, -1);
    $parents[] = 'condition_type';
    if (!$value = NestedArray::getValue($form_state->getUserInput(), $parents)) {
      $form_state->setError(NestedArray::getValue($form, $parents), 'Condition type required.');
    }
  }

  public function addElementCondition(array &$form, FormStateInterface $form_state) {
    //@todo: reorder conditions to account for drupal core issue.
    $button = $form_state->getTriggeringElement();
    // Save condition weight.
    $condition_values = NestedArray::getValue($form_state->getUserInput(), array_slice($button['#parents'], 0, -2));
    if (isset($condition_values['condition_items'])) {
      $this->attachTableConditionWeight($condition_values['condition_items']);
    }

    $type = NestedArray::getValue($form_state->getUserInput(), array_slice($button['#parents'], 0, -1))['condition_type'];
    $this->addCondition($this->conditionManager->createInstance($type));
    $form_state->setRebuild();
  }

  public function addElementConditionAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -2));
  }

  public function removeElementCondition(array &$form, FormStateInterface $form_state) {

    $button = $form_state->getTriggeringElement();

    list($action, $name) = explode('__', $button['#name']);

    // Save condition weight.
    $condition_values = NestedArray::getValue($form_state->getUserInput(), array_slice($button['#parents'], 0, -3));
    if (isset($condition_values['condition_items'])) {
      $this->attachTableConditionWeight($condition_values['condition_items']);
    }

    $variation = $this->entity->getVariation($this->id());
    $variation->removeCondition($name);
    $form_state->setRebuild();

  }

  public function removeElementConditionAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -3));
  }

  public function attachTableReactionWeight($values) {
    foreach ($this->getReactions() as $reaction) {
      if (isset($values[$reaction->id()]['weight'])) {
        $reaction->setWeight($values[$reaction->id()]['weight']);
      }
    }
    $this->sortReactions();
  }

  public function addElementReaction(array &$form, FormStateInterface $form_state) {
    //@todo: reorder reactions to account for drupal core issue.
    $button = $form_state->getTriggeringElement();

    // Save reaction weight.
    $reaction_values = NestedArray::getValue($form_state->getUserInput(), array_slice($button['#parents'], 0, -2));
    if (isset($reaction_values['reaction_items'])) {
      $this->attachTableReactionWeight($reaction_values['reaction_items']);
    }

    $variation_set = $this->entity;
    $this->addReaction($this->reactionManager->createInstance('block'));
    $variation_set->addVariation($this);
    $form_state->setRebuild();
  }


  public function addElementReactionAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -2));
  }

  public function removeElementReaction(array &$form, FormStateInterface $form_state) {

    $button = $form_state->getTriggeringElement();

    list($action, $name) = explode('__', $button['#name']);

    // Save reaction weight.
    $reaction_values = NestedArray::getValue($form_state->getUserInput(), array_slice($button['#parents'], 0, -3));
    if (isset($reaction_values['reaction_items'])) {
      $this->attachTableReactionWeight($reaction_values['reaction_items']);
    }

    $variation_set = $this->entity;
    $variation = $variation_set->getVariation($this->id());
    $variation->removeReaction($name);
    $form_state->setRebuild();

  }

  public function removeElementReactionAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    // Go one level up in the form, to the widgets container.
    return NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -3));
  }

}
